@@ -744,29 +744,23 @@ get_cursor(void)
 static void
 set_bit_panel(void)
 {
-    int bit_str_len, i;
-    int MP[MP_SIZE];
-    char bit_str[MAXLINE], label[MAXLINE];
-
-    if (display_is_usable_number(&v->display, MP) || !is_integer(MP)) {
-        gtk_widget_set_sensitive(X->bit_panel, FALSE);
+    int i;
+    const gchar *label;
+    guint64 value;
+    
+    if (!display_get_unsigned_integer(&v->display, &value))
+    {
+        gtk_widget_set_sensitive(X->bit_panel, FALSE);        
         return;
     }
-    make_fixed(bit_str, MAXLINE, MP, BIN, MAXLINE, FALSE);
-    bit_str_len = strlen(bit_str);
-    if (bit_str_len <= MAXBITS) {
-        gtk_widget_set_sensitive(X->bit_panel, TRUE);
-        
-        for (i = 0; i < MAXBITS; i++) {
-            if (i < bit_str_len) {
-                SNPRINTF(label, MAXLINE, " %c", bit_str[bit_str_len-i-1]);
-            } else {
-                SNPRINTF(label, MAXLINE, " 0");
-            }
-            gtk_label_set_text(GTK_LABEL(X->bits[MAXBITS - i - 1]), label);
-        }
-    } else {
-        gtk_widget_set_sensitive(X->bit_panel, FALSE);
+    gtk_widget_set_sensitive(X->bit_panel, TRUE);
+
+    for (i = 0; i < MAXBITS; i++) {
+        if (value & (1LL << (MAXBITS-i-1)))
+            label = " 1";
+        else
+            label = " 0";
+        gtk_label_set_text(GTK_LABEL(X->bits[i]), label);
     }
 }
 
@@ -1245,50 +1239,26 @@ base_cb(GtkWidget *widget)
 static void
 help_display(void)
 {
+    GdkScreen *screen;
     GError *error = NULL;
-    char *command;
-    const char *lang;
-    char *uri = NULL;
-    GdkScreen *gscreen;
-    int i;
-    
-    const char * const * langs = g_get_language_names ();
-    
-    for (i = 0; langs[i]; i++) {
-        lang = langs[i];
-        if (strchr (lang, '.')) {
-            continue;
-        }
-        
-        uri = g_build_filename(PACKAGE_DATA_DIR,
-                               "/gnome/help/gcalctool/",
-                               lang,
-                               "/gcalctool.xml",
-                               NULL);
-        
-        if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
-            break;
-        }
-    }
-    
-    command = g_strconcat ("gnome-open ghelp://", uri, NULL);
-    gscreen = gdk_screen_get_default();
-    gdk_spawn_command_line_on_screen (gscreen, command, &error);
-    if (error) {
+ 
+    screen = gtk_widget_get_screen (GTK_WIDGET (X->kframe));
+    gtk_show_uri (screen, "ghelp:gcalctool", gtk_get_current_event_time (), &error);
+ 
+    if (error != NULL)
+    {
         GtkWidget *d;
+        d = gtk_message_dialog_new (GTK_WINDOW (X->kframe), 
+                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                    GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                    "%s", "Unable to open help file");
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (d),
+                                                  "%s", error->message);
+        g_signal_connect (d, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+        gtk_window_present (GTK_WINDOW (d));
         
-        d = gtk_message_dialog_new(NULL,
-                           GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                           GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                           error->message);
-        gtk_dialog_run(GTK_DIALOG(d));
-        gtk_widget_destroy(d);
-        g_error_free(error);
-        error = NULL;
+        g_error_free (error);
     }
-    
-    g_free (command);
-    g_free (uri);
 }
 
 
@@ -1607,9 +1577,8 @@ request_change_mode()
     gtk_widget_show_all(dialog);
     response = gtk_dialog_run(GTK_DIALOG(dialog));
     
-    // FIXME: Save this in GConf
-    X->warn_change_mode = !gtk_toggle_button_get_active(
-                             GTK_TOGGLE_BUTTON(request_check));
+    X->warn_change_mode = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(request_check));
+    set_boolean_resource(R_CHMODE, X->warn_change_mode);
 
     gtk_widget_destroy(dialog);
 
@@ -1621,34 +1590,32 @@ request_change_mode()
 static gboolean
 bit_toggle_cb(GtkWidget *event_box, GdkEventButton *event)
 {
-    double number;
-    unsigned long long lval;
-    int n, MP1[MP_SIZE], index;
+    guint64 value;
+    int index;
+    const gchar *text;
+    char buf[MAX_DISPLAY];
+    int MP[MP_SIZE];
+    
+    index = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(event_box), "bit_index"));
+    assert(display_get_unsigned_integer(&v->display, &value));
+    value ^= (1LL << (MAXBITS - index - 1));
+    
+    /* FIXME: Convert to since we don't support setting MP numbers from 64 bit integers */
+    SNPRINTF(buf, MAX_DISPLAY, "%llu", value);
+    MPstr_to_num(buf, DEC, MP);
 
-    index = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(event_box),
-                                              "bit_index"));
-    n = MAXBITS - index - 1;
-
-    int ret = display_is_usable_number(&v->display, display_get_answer(&v->display));
-    assert(!ret);
-    mp_set_from_mp(display_get_answer(&v->display), MP1);
-
-    number = mp_cast_to_double(MP1);
-    lval = (long long) number;
-
-    if (lval & (1LL << n)) {
-        lval &= ~(1LL << n);
-        gtk_label_set_text(GTK_LABEL(X->bits[index]), " 0");
-    } else {
-        lval |=  (1LL << n);
-        gtk_label_set_text(GTK_LABEL(X->bits[index]), " 1");
-    }
-    number = (double) lval;
-
-    mp_set_from_double(number, display_get_answer(&v->display));
-    display_set_string(&v->display, "Ans", -1);
+    /* FIXME: Set as string as display_set_number doesn't store correctly */
+    make_number(buf, MAX_DISPLAY, MP, v->base, FALSE);
+    display_set_string(&v->display, buf, -1);
     display_refresh(&v->display);
 
+    text = gtk_label_get_text(GTK_LABEL(X->bits[index]));
+    if (strcmp(text, " 0") == 0)
+        text = " 1";
+    else
+        text = " 0";        
+    gtk_label_set_text(GTK_LABEL(X->bits[index]), text);
+
     return (TRUE);
 }
 
@@ -1814,6 +1781,16 @@ hyp_cb(GtkWidget *widget)
 }
 
 
+/*ARGSUSED*/
+void
+trig_cb(GtkWidget *widget)
+{
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+        do_trigtype((enum trig_type) g_object_get_data(G_OBJECT(widget),
+                                                       "trig_mode"));
+}
+
+
 /*ARGSUSED*/
 void
 inv_cb(GtkWidget *widget)
@@ -2409,6 +2386,7 @@ create_kframe()
     CONNECT_SIGNAL(menu_item_select_cb);
     CONNECT_SIGNAL(menu_item_deselect_cb);
     CONNECT_SIGNAL(mode_radio_cb);
+    CONNECT_SIGNAL(trig_cb);
     CONNECT_SIGNAL(inv_cb);
     CONNECT_SIGNAL(hyp_cb);
     CONNECT_SIGNAL(base_cb);
@@ -2744,6 +2722,9 @@ ui_load(void)
 
     read_cfdefs();
     
+    if(!get_boolean_resource(R_CHMODE, &X->warn_change_mode))
+        X->warn_change_mode = TRUE;
+    
     /* Create main gcalctool window. */
     create_kframe();
     
@@ -2790,8 +2771,6 @@ ui_load(void)
 void
 ui_start(void)
 {
-    X->warn_change_mode = TRUE; // FIXME: Load from GConf
-    
     ui_set_base(v->base);
     ui_set_trigonometric_mode(v->ttype);
     ui_set_numeric_mode(v->dtype);
